@@ -8,8 +8,12 @@
  *  against the fault so an error message can be emitted instead of
  *  a core dump :-)
  *
+ * @addtogroup autogen
+ * @{
+ */
+/*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2014 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -273,7 +277,7 @@ ag_scm_hide_email(SCM display, SCM eaddr)
         (strlen(end_adr) * HTML_DEC_DIGIT_LEN)
         + (size_t)st_len + HIDE_EMAIL_END_FMT_LEN + strlen(disp));
 
-    char *  res  = ag_scribble(str_size);
+    char *  res  = scribble_get(str_size);
     char *  scan = res;
 
     memcpy(scan, HIDE_EMAIL_START_STR, (size_t)st_len);
@@ -334,7 +338,9 @@ ag_scm_format_arg_count(SCM fmt)
 
     return AG_SCM_INT2SCM(ct);
 }
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
