@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for agen5
- * Generated Wed Oct 16 13:46:35 PDT 2013
+ * Generated Sat May 17 07:50:07 PDT 2014
  */
 #ifndef AGEN5_PROTO_H_GUARD
 #define AGEN5_PROTO_H_GUARD 1
@@ -87,10 +87,10 @@ skip_expr(char const * pzSrc, size_t len);
 /*
  *  Extracted from autogen.c
  */
-static void
-ag_abend_at(char const * pzMsg
+static _Noreturn void
+ag_abend_at(char const * msg
 #ifdef DEBUG_ENABLED
-            , char const * pzFile, int line
+            , char const * fname, int line
 #endif
     );
 
