@@ -9,7 +9,7 @@
  */
 /*
  * This file is part of AutoGen.
- * AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
+ * AutoGen Copyright (C) 1992-2014 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -124,7 +124,7 @@ do_stdout_tpl(templ_t * tpl)
         return;
 
     default:
-        fprintf(stdout, DO_STDOUT_TPL_BADR, oops_pfx);
+        fserr(AUTOGEN_EXIT_FS_ERROR, DO_STDOUT_TPL_BADR, oops_pfx);
 
     case FAILURE:
         exit(EXIT_FAILURE);
